from .addtc.manager import manage as addtc_manage
from .config.manager import manage as config_manage
from .create.manager import manage as create_manage
from .fetch.manager import manage as fetch_manage
from .run.manager import manage as run_manage
